@@ -12,3 +12,8 @@ pref("accessibility.typeaheadfindsea.linksonly", true);
 // disable image resizing
 pref("browser.enable_automatic_image_resizing", false);
 
+// enable password manager
+// need to include those prefs since xulrunner doesn't include them
+pref("signon.rememberSignons", true);
+pref("signon.expireMasterPassword", false);
+pref("signon.SignonFileName", "signons.txt");
