@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-embed-shell.c,v 1.34 2005/10/02 22:02:57 chpe Exp $
+ *  $Id: ephy-embed-shell.c,v 1.37 2006/02/02 17:27:40 chpe Exp $
  */
 
 #include "config.h"
@@ -31,6 +31,10 @@
 #include "downloader-view.h"
 #include "ephy-encodings.h"
 #include "ephy-debug.h"
+#include "ephy-adblock-manager.h"
+
+#include <glib/gi18n.h>
+#include <gtk/gtkmessagedialog.h>
 
 #define EPHY_EMBED_SHELL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_EMBED_SHELL, EphyEmbedShellPrivate))
 
@@ -41,6 +45,8 @@ struct _EphyEmbedShellPrivate
 	EphyFaviconCache *favicon_cache;
 	EphyEmbedSingle *embed_single;
 	EphyEncodings *encodings;
+	EphyAdBlockManager *adblock_manager;
+	guint single_initialised : 1;
 };
 
 enum
@@ -136,6 +142,13 @@ ephy_embed_shell_finalize (GObject *object)
 		g_object_unref (G_OBJECT (shell->priv->embed_single));
 	}
 
+	if (shell->priv->adblock_manager != NULL)
+	{
+		LOG ("Unref adblock manager");
+		g_object_unref (shell->priv->adblock_manager);
+		shell->priv->adblock_manager = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -189,15 +202,52 @@ ephy_embed_shell_get_downloader_view (EphyEmbedShell *shell)
 	return G_OBJECT (shell->priv->downloader_view);
 }
 
-static GObject *
-impl_get_embed_single (EphyEmbedShell *shell)
+GObject *
+ephy_embed_shell_get_downloader_view_nocreate (EphyEmbedShell *shell)
 {
 	g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
 
-	if (shell->priv->embed_single == NULL)
+	return (GObject *) shell->priv->downloader_view;
+}
+
+static GObject *
+impl_get_embed_single (EphyEmbedShell *shell)
+{
+	EphyEmbedShellPrivate *priv;
+
+	g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+
+	priv = shell->priv;
+
+	if (priv->embed_single != NULL &&
+	    !priv->single_initialised)
 	{
-		shell->priv->embed_single = EPHY_EMBED_SINGLE
+		g_warning ("ephy_embed_shell_get_embed_single called while the single is being initialised!\n");
+		return G_OBJECT (priv->embed_single);
+	}
+
+	if (priv->embed_single == NULL)
+	{
+		priv->embed_single = EPHY_EMBED_SINGLE
 			(ephy_embed_factory_new_object (EPHY_TYPE_EMBED_SINGLE));
+
+		if (!ephy_embed_single_init (priv->embed_single))
+		{
+			GtkWidget *dialog;
+
+			dialog = gtk_message_dialog_new
+					(NULL,
+					 GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_ERROR,
+					 GTK_BUTTONS_CLOSE,
+					 _("Epiphany can't be used now. "
+							 "Mozilla initialization failed."));
+			gtk_dialog_run (GTK_DIALOG (dialog));
+
+			exit (0);
+		}
+
+		priv->single_initialised = TRUE;
 	}
 
 	return G_OBJECT (shell->priv->embed_single);
@@ -284,3 +334,25 @@ ephy_embed_shell_get_default (void)
 {
 	return embed_shell;
 }
+
+/**
+ * ephy_embed_shell_get_adblock_manager:
+ * @shell: the #EphyEmbedShell
+ *
+ * Returns the adblock manager.
+ *
+ * Return value: the adblock manager
+ **/
+GObject *
+ephy_embed_shell_get_adblock_manager (EphyEmbedShell *shell)
+{
+	g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
+
+	if (shell->priv->adblock_manager == NULL)
+	{
+		shell->priv->adblock_manager = g_object_new (EPHY_TYPE_ADBLOCK_MANAGER, NULL);
+	}
+
+	return G_OBJECT (shell->priv->adblock_manager);
+}
+
