@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyUtils.cpp,v 1.13 2005/07/24 21:37:01 chpe Exp $
+ *  $Id: EphyUtils.cpp,v 1.17 2006/01/17 13:58:19 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -30,6 +30,7 @@
 
 #include <nsIIOService.h>
 #include <nsIURI.h>
+#include <nsIFile.h>
 #include <nsIDOMWindow.h>
 #include <nsIServiceManager.h>
 #undef MOZILLA_INTERNAL_API
@@ -75,14 +76,23 @@ EphyUtils::NewURI (nsIURI **result,
 		   nsIURI *baseURI)
 {
 	nsresult rv;
-
 	nsCOMPtr<nsIIOService> ioService;
 	rv = EphyUtils::GetIOService (getter_AddRefs (ioService));
 	NS_ENSURE_SUCCESS (rv, rv);
 
-	rv = ioService->NewURI (spec, charset, baseURI, result);
+	return ioService->NewURI (spec, charset, baseURI, result);
+}
 
-	return rv;
+nsresult
+EphyUtils::NewFileURI (nsIURI **result,
+		       nsIFile *spec)
+{
+	nsresult rv;
+	nsCOMPtr<nsIIOService> ioService;
+	rv = EphyUtils::GetIOService (getter_AddRefs (ioService));
+	NS_ENSURE_SUCCESS (rv, rv);
+
+	return ioService->NewFileURI (spec, result);
 }
 
 GtkWidget *
@@ -122,7 +132,10 @@ EphyUtils::FindGtkParent (nsIDOMWindow *aDOMWindow)
 	GtkWidget *embed = FindEmbed (aDOMWindow);
 	NS_ENSURE_TRUE (embed, nsnull);
 
-	return gtk_widget_get_toplevel (GTK_WIDGET (embed));
+	GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (embed));
+	if (!GTK_WIDGET_TOPLEVEL (toplevel)) return nsnull;
+
+	return toplevel;
 }
 
 nsresult
@@ -130,6 +143,12 @@ EphyUtils::CollatePrintSettings (EmbedPrintInfo *info,
 				 nsIPrintSettings *options,
 				 gboolean preview)
 {
+	NS_ENSURE_ARG (options);
+
+	/* FIXME: for CUPS printers, print directly instead of to a tmp file? */
+	const static PRUnichar pName[] = { 'P', 'o', 's', 't', 'S', 'c', 'r', 'i', 'p', 't', '/', 'd', 'e', 'f', 'a', 'u', 'l', 't', '\0' };
+	options->SetPrinterName(nsEmbedString(pName).get());
+
 	const static int frame_types[] = {
 		nsIPrintSettings::kFramesAsIs,
 		nsIPrintSettings::kSelectedFrame,
@@ -155,7 +174,7 @@ EphyUtils::CollatePrintSettings (EmbedPrintInfo *info,
 	
 	const GnomePrintUnit *unit, *inch, *mm;
 	double value;
-	
+
 	mm = gnome_print_unit_get_by_abbreviation ((const guchar *) "mm");
 	inch = gnome_print_unit_get_by_abbreviation ((const guchar *) "in");
 	g_assert (mm != NULL && inch != NULL);
@@ -225,9 +244,13 @@ EphyUtils::CollatePrintSettings (EmbedPrintInfo *info,
 
 	options->SetPrintToFile (PR_FALSE);
 	
-	if (!preview)
+	if (preview)
 	{
-		char *cmd, *base;
+		options->SetPrintToFile (PR_FALSE);
+	}
+	else
+	{
+		char *base;
 		const char *temp_dir;
 		
 		temp_dir = ephy_file_tmp_dir ();
@@ -235,16 +258,12 @@ EphyUtils::CollatePrintSettings (EmbedPrintInfo *info,
 		info->tempfile = ephy_file_tmp_filename (base, "ps");
 		g_free (base);
 		
-		/* use cat instead of print to file to avoid fflush to ensure
-		 * the file has been written completely and we don't need to
-		 * select a printer (i.e. should be printing backend independent)
-		 */
+		if (info->tempfile == NULL) return NS_ERROR_FAILURE;
 		
-		cmd = g_strconcat ("cat > ", info->tempfile, NULL);
-		NS_CStringToUTF16 (nsEmbedCString(cmd),
+		NS_CStringToUTF16 (nsEmbedCString(info->tempfile),
 				   NS_CSTRING_ENCODING_UTF8, tmp);
-		options->SetPrintCommand (tmp.get());
-		g_free (cmd);
+		options->SetPrintToFile (PR_TRUE);
+		options->SetToFileName (tmp.get());
 	}
 
 	/* paper size */
@@ -266,13 +285,43 @@ EphyUtils::CollatePrintSettings (EmbedPrintInfo *info,
 	{
 		options->SetPaperHeight (value);	
 	}
-	
-	char *string;
+
+	/* Mozilla bug https://bugzilla.mozilla.org/show_bug.cgi?id=307404
+	 * means that we cannot actually use any paper sizes except mozilla's
+	 * builtin list, and we must refer to them *by name*!
+	 */
+#ifndef HAVE_GECKO_1_9
+	/* Gnome-Print names some papers differently than what moz understands */
+	static const struct
+	{
+		const char *gppaper;
+		const char *mozpaper;
+	}
+	paper_table [] =
+	{
+		{ "USLetter", "Letter" },
+		{ "USLegal", "Legal" }
+	};
+#endif /* !HAVE_GECKO_1_9 */
 
 	/* paper name */
-	string = (char *) gnome_print_config_get (info->config,
+	char *string = (char *) gnome_print_config_get (info->config,
 						  (const guchar *) GNOME_PRINT_KEY_PAPER_SIZE);
-	NS_CStringToUTF16 (nsEmbedCString(string),
+	const char *paper = string;
+
+#ifndef HAVE_GECKO_1_9
+	for (PRUint32 i = 0; i < G_N_ELEMENTS (paper_table); i++)
+	{
+		if (string != NULL &&
+		    g_ascii_strcasecmp (paper_table[i].gppaper, string) == 0)
+		{
+			paper = paper_table[i].mozpaper;
+			break;
+		}
+	}
+#endif /* !HAVE_GECKO_1_9 */
+
+	NS_CStringToUTF16 (nsEmbedCString(paper),
 			   NS_CSTRING_ENCODING_UTF8, tmp);
 	options->SetPaperName (tmp.get());
 	g_free (string);
