@@ -38,7 +38,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.71 2005/05/25 22:08:01 chpe Exp $
+ * $Id: MozDownload.cpp,v 1.78 2006/01/23 21:35:10 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -53,7 +53,6 @@
 #include "MozDownload.h"
 #include "EphyUtils.h"
 
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n.h>
 
 #include <nsIIOService.h>
@@ -72,6 +71,10 @@
 
 #include <stdlib.h>
 
+#ifdef HAVE_GECKO_1_8
+#include "EphyBadCertRejector.h"
+#endif
+
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
 
 MozDownload::MozDownload() :
@@ -93,20 +96,28 @@ MozDownload::~MozDownload()
 }
 
 #ifdef HAVE_GECKO_1_8
-NS_IMPL_ISUPPORTS3(MozDownload, nsIWebProgressListener, nsIWebProgressListener2, nsITransfer)
+NS_IMPL_ISUPPORTS4 (MozDownload,
+		    nsIWebProgressListener,
+		    nsIWebProgressListener2,
+		    nsITransfer,
+		    nsIInterfaceRequestor)
 #else
-NS_IMPL_ISUPPORTS3(MozDownload, nsIWebProgressListener, nsIDownload, nsITransfer)
+NS_IMPL_ISUPPORTS3 (MozDownload,
+		    nsIWebProgressListener,
+		    nsIDownload,
+		    nsITransfer)
 #endif
 
 #ifdef HAVE_GECKO_1_8
 nsresult
 MozDownload::InitForEmbed (nsIURI *aSource, nsIURI *aTarget, const nsAString &aDisplayName,
-		           nsIMIMEInfo *aMIMEInfo, PRTime aStartTime, nsICancelable *aCancelable,
-		           MozillaEmbedPersist *aEmbedPersist, PRInt64 aMaxSize)
+		           nsIMIMEInfo *aMIMEInfo, PRTime aStartTime, nsILocalFile *aTempFile,
+			   nsICancelable *aCancelable, MozillaEmbedPersist *aEmbedPersist,
+			   PRInt64 aMaxSize)
 {
 	mEmbedPersist = aEmbedPersist;
 	mMaxSize = aMaxSize;
-	return Init (aSource, aTarget, aDisplayName, aMIMEInfo, aStartTime, aCancelable);
+	return Init (aSource, aTarget, aDisplayName, aMIMEInfo, aStartTime, aTempFile, aCancelable);
 }
 #else
 nsresult
@@ -121,13 +132,14 @@ MozDownload::InitForEmbed (nsIURI *aSource, nsIURI *aTarget, const PRUnichar *aD
 #endif
 
 #ifdef HAVE_GECKO_1_8
-/* void init (in nsIURI aSource, in nsIURI aTarget, in AString aDisplayName, in nsIMIMEInfo aMIMEInfo, in PRTime startTime, in nsICancelable aCancelable); */
+/* void init (in nsIURI aSource, in nsIURI aTarget, in AString aDisplayName, in nsIMIMEInfo aMIMEInfo, in PRTime startTime, in nsILocalFile aTempFile, in nsICancelable aCancelable); */
 NS_IMETHODIMP
 MozDownload::Init (nsIURI *aSource,
 		   nsIURI *aTarget,
 		   const nsAString &aDisplayName,
 		   nsIMIMEInfo *aMIMEInfo,
 		   PRTime aStartTime,
+		   nsILocalFile *aTempFile,
 		   nsICancelable *aCancelable)
 #else
 /* void init (in nsIURI aSource, in nsIURI aTarget, in wstring aDisplayName, in nsIMIMEInfo aMIMEInfo, in long long startTime, in nsIWebBrowserPersist aPersist); */
@@ -367,8 +379,34 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 	}
 
 	/* We will get this even in the event of a cancel */
-	if (aStateFlags & STATE_STOP)
+	/* Due to a mozilla bug [https://bugzilla.mozilla.org/show_bug.cgi?id=304353],
+	 * we'll only get STATE_STOP if we're driven from external app handler; elsewhere
+	 * we get STATE_STOP | STATE_IS_NETWORK | STATE_IS_REQUEST. So check first if 
+	 * STATE_IS_REQUEST is set.
+	 */
+	/* Be careful that download is only completed when STATE_IS_NETWORK is set
+ 	 * and many lonely STOP events may be triggered before.
+	 */
+#ifdef GNOME_ENABLE_DEBUG
+{
+	nsEmbedCString spec;
+	if (mSource) mSource->GetSpec(spec);
+
+	LOG ("url %s, status %x, state %x (is-stop:%s, is-network:%s, is-request:%s)",
+	     spec.get(), aStatus, aStateFlags,
+	     aStateFlags & STATE_STOP ? "t" : "f",
+	     aStateFlags & STATE_IS_NETWORK ? "t" : "f",
+	     aStateFlags & STATE_IS_REQUEST ? "t" : "f");
+}
+#endif
+
+	if (((aStateFlags & STATE_IS_REQUEST) &&
+	     (aStateFlags & STATE_IS_NETWORK) &&
+	     (aStateFlags & STATE_STOP)) ||
+	    aStateFlags == STATE_STOP)
 	{
+		LOG ("STATE_STOP");
+
 		/* Keep us alive */
 		nsCOMPtr<nsITransfer> kungFuDeathGrip(this);
 
@@ -555,6 +593,36 @@ MozDownload::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *aReques
 	return NS_OK;
 }
 
+#ifdef HAVE_GECKO_1_8
+
+/* void getInterface (in nsIIDRef uuid, [iid_is (uuid), retval] out nsQIResult result); */
+NS_IMETHODIMP
+MozDownload::GetInterface(const nsIID & uuid, void * *result)
+{
+	if (uuid.Equals (NS_GET_IID (nsIBadCertListener)) &&
+	    mEmbedPersist)
+	{
+		EphyEmbedPersistFlags flags;
+
+		g_object_get (mEmbedPersist, "flags", &flags, NULL);
+
+		if (flags & EPHY_EMBED_PERSIST_NO_CERTDIALOGS)
+		{
+			nsIBadCertListener *badCertRejector = new EphyBadCertRejector ();
+			if (!badCertRejector) return NS_ERROR_OUT_OF_MEMORY;
+
+			*result = badCertRejector;
+			NS_ADDREF (badCertRejector);
+
+			return NS_OK;
+		}
+	}
+
+	return NS_ERROR_NO_INTERFACE;
+}
+
+#endif /* HAVE_GECKO_1_8 */
+
 void
 MozDownload::Cancel()
 {
@@ -639,7 +707,7 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 	/* FIXME is that still true? */
 #ifdef HAVE_GECKO_1_8
 	rv = downloader->InitForEmbed (inOriginalURI, destURI, fileDisplayName,
-				       nsnull, timeNow, webPersist, embedPersist, aMaxSize);
+				       nsnull, timeNow, nsnull, webPersist, embedPersist, aMaxSize);
 	NS_ENSURE_SUCCESS (rv, rv);
 
 	rv = webPersist->SetProgressListener (downloader);
@@ -704,50 +772,22 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 static char*
 GetFilePath (const char *filename)
 {
-	char *path = NULL, *download_dir, *expanded;
+	const char *home_dir;
+	char *download_dir, *path;
 
-	download_dir = eel_gconf_get_string (CONF_STATE_DOWNLOAD_DIR);
+	download_dir = ephy_file_get_downloads_dir ();
 
-	if (download_dir && strcmp (download_dir, "Downloads") == 0)
+	if (ephy_ensure_dir_exists (download_dir, NULL))
 	{
-		g_free (download_dir);
-		download_dir = ephy_file_downloads_dir ();
+		path = g_build_filename (download_dir, filename, NULL);
 	}
-  	else if (download_dir && strcmp (download_dir, "Desktop") == 0)
+	else
 	{
-		g_free (download_dir);
-		download_dir = ephy_file_desktop_dir ();
-	}  
-	else if (download_dir)
-	{
-		char *converted_dp;
-
-		converted_dp = g_filename_from_utf8 (download_dir, -1, NULL, NULL, NULL);
-		g_free (download_dir);
-		download_dir = converted_dp;
+		home_dir = g_get_home_dir ();
+		path = g_build_filename (home_dir ? home_dir : "/", filename, NULL);
 	}
-
-	if (download_dir == NULL)
-	{
-		/* Emergency download destination */
-		download_dir = g_strdup (g_get_home_dir ());
-	}
-
-	g_return_val_if_fail (download_dir != NULL, FALSE);
-
-	expanded = gnome_vfs_expand_initial_tilde (download_dir);
-	if (ephy_ensure_dir_exists (expanded))
-	{
-		path = g_build_filename (expanded, filename, NULL);
-	}
-	g_free (expanded);
 	g_free (download_dir);
 	
-	if (path == NULL)
-	{
-		path = g_build_filename (g_get_home_dir (), filename, NULL);
-	}
-
 	return path;
 }
 
@@ -755,7 +795,7 @@ static const char*
 file_is_compressed (const char *filename)
 {
         int i;
-        const char *compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
+        static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
 
         for (i = 0; compression[i] != NULL; i++)
         {
@@ -777,7 +817,7 @@ parse_extension (const char *filename)
         if (compression != NULL)
         {
                 int i;
-                const char *extensions[] = {"tar", "ps", "xcf", "dvi", "txt", "text", NULL};
+                static const char * const extensions[] = {"tar", "ps", "xcf", "dvi", "txt", "text", NULL};
 
                 for (i = 0; extensions[i] != NULL; i++)
                 {
