@@ -17,13 +17,17 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: PrintingPromptService.cpp,v 1.22 2005/07/23 21:31:19 chpe Exp $
+ *  $Id: PrintingPromptService.cpp,v 1.23 2006/02/02 22:25:18 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
 #include "config.h"
 
+#include "PrintingPromptService.h"
+#include "EphyUtils.h"
+#include "AutoJSContextStack.h"
+
 #include <gtk/gtkdialog.h>
  
 #include <libgnomeprintui/gnome-print-dialog.h>
@@ -31,8 +35,6 @@
 #include "print-dialog.h"
 #include "ephy-embed.h"
 #include "ephy-command-manager.h"
-#include "EphyUtils.h"
-#include "PrintingPromptService.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-debug.h"
@@ -72,6 +74,11 @@ NS_IMETHODIMP GPrintingPromptService::ShowPrintDialog(nsIDOMWindow *parent, nsIW
 		return NS_ERROR_ABORT;
 	}
 
+	nsresult rv;
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	EphyEmbed *embed = EPHY_EMBED (EphyUtils::FindEmbed (parent));
 	NS_ENSURE_TRUE (embed, NS_ERROR_FAILURE);
 
@@ -136,7 +143,8 @@ NS_IMETHODIMP GPrintingPromptService::ShowProgress(nsIDOMWindow *parent, nsIWebB
 }
 
 /* void showPageSetup (in nsIDOMWindow parent, in nsIPrintSettings printSettings, in nsIObserver printObserver); */
-NS_IMETHODIMP GPrintingPromptService::ShowPageSetup(nsIDOMWindow *parent, nsIPrintSettings *printSettings, 
+NS_IMETHODIMP GPrintingPromptService::ShowPageSetup(nsIDOMWindow *parent,
+						    nsIPrintSettings *printSettings, 
 						    nsIObserver *printObserver)
 {
 	EphyDialog *dialog;
@@ -148,6 +156,10 @@ NS_IMETHODIMP GPrintingPromptService::ShowPageSetup(nsIDOMWindow *parent, nsIPri
 		return rv;
 	}
 
+	AutoJSContextStack stack;
+	rv = stack.Init ();
+	if (NS_FAILED (rv)) return rv;
+
 	dialog = ephy_print_setup_dialog_new ();
 	ephy_dialog_set_modal (dialog, TRUE);
 
